#include "nunchuck.h"

///////////////////////////////////////////////////////////////////////////////
//
// NUNCHUCK functions
//
///////////////////////////////////////////////////////////////////////////////

// Communication with the Nunchuk consists of two phases:
//
// -->Phase 1: initialization phase (executed once) in which specific data
// are written to the NunChuk;
// Essentially initialization consists of two write transactions,
// each of which writes a single byte to a register internal to
// the I2C slave ( reg[0xf0] = 0x55, reg[0xfb] = 0x00 ).
//
// -->Phase 2: repeated read phase in which six data bytes are read
// again and again; each read phase consists of two transactions –
// a write transaction which sets the read address to zero, and a
// read transaction.
// NOTE: NunChuck uses 'fast mode' for I2C communiations!

HAL_StatusTypeDef NUNCHUCK::Write_To_NunChuck(uint16_t DevAddress, uint8_t *pData, uint16_t len)
{
  HAL_StatusTypeDef returnValue;

  // send data over the I2C bus
  returnValue = HAL_I2C_Master_Transmit(_hi2c, DevAddress, pData, len, HAL_MAX_DELAY);
  if (returnValue != HAL_OK)
    return returnValue;

  return HAL_OK;
}

HAL_StatusTypeDef NUNCHUCK::Read_From_NunChuck(uint16_t DevAddress, uint8_t *pData, uint16_t len)
{
  HAL_StatusTypeDef returnValue;

  // retrieve data over the I2C;
  returnValue = HAL_I2C_Master_Receive(_hi2c, DevAddress, pData, len, HAL_MAX_DELAY);

  return returnValue;
}

void NUNCHUCK::print_data_initially(void)
{
  // Note: this function should be called once only;
  LCD_PutStr(32,32,  (char *)"This is I2C example", DEFAULT_FONT, C_WHITE, C_BLACK);
  LCD_PutStr(32,48,  (char *)"Data from NunChuck:", DEFAULT_FONT, C_WHITE, C_BLACK);
  LCD_PutStr(32,64,  (char *)"joyX =", DEFAULT_FONT, C_WHITE, C_BLACK);
  LCD_PutStr(32,80,  (char *)"joyY =", DEFAULT_FONT, C_WHITE, C_BLACK);
  LCD_PutStr(32,96,  (char *)"accX =", DEFAULT_FONT, C_WHITE, C_BLACK);
  LCD_PutStr(32,112, (char *)"accY =", DEFAULT_FONT, C_WHITE, C_BLACK);
  LCD_PutStr(32,128, (char *)"accZ =", DEFAULT_FONT, C_WHITE, C_BLACK);
  LCD_PutStr(32,144, (char *)"Z    =", DEFAULT_FONT, C_WHITE, C_BLACK);
  LCD_PutStr(32,160, (char *)"C    =", DEFAULT_FONT, C_WHITE, C_BLACK);
}

void NUNCHUCK::phase1_init(void)
{
  // this function should be called once only;

  _master_transmit_buffer[0] = 0xF0; // at address 0xF0 of NunChuck write:
  _master_transmit_buffer[1] = 0x55; // data 0x55
  Write_To_NunChuck(_address_slave1, _master_transmit_buffer, 2);
  HAL_Delay(10);

  _master_transmit_buffer[0] = 0xFB; // at address 0xFB of NunChuck write:
  _master_transmit_buffer[1] = 0x00; // data 0x00
  Write_To_NunChuck(_address_slave1, _master_transmit_buffer, 2);
  HAL_Delay(10);
}

void NUNCHUCK::phase2_read(void)
{
  // this is called repeatedly to realize continued polling of NunChuck;

  // write one byte;
  _master_transmit_buffer[0] = 0x00; // value;
  Write_To_NunChuck(_address_slave1, _master_transmit_buffer, 1);
  HAL_Delay(10);

  // read 6 bytes;
  Read_From_NunChuck(_address_slave2, _slave_receive_buffer, 6);
  HAL_Delay(10);
}

void NUNCHUCK::translate_data(void)
{
  // this function gets the data received from the NunChuck
  // and translates so that it can be used; you need to read
  // the lab description and the datasheet/manual of the NunChuck to
  // understand this translation;
  int byte5 = _slave_receive_buffer[5];
  _joy_x_axis = _slave_receive_buffer[0];
  _joy_y_axis = _slave_receive_buffer[1];
  _accel_x_axis = (_slave_receive_buffer[2] << 2);
  _accel_y_axis = (_slave_receive_buffer[3] << 2);
  _accel_z_axis = (_slave_receive_buffer[4] << 2);
  _z_button = 0;
  _c_button = 0;

  // byte _slave_receive_buffer[5] contains bits for z and c buttons
  // it also contains the least significant bits for the accelerometer data;
  if ((byte5 >> 0) & 1)
    _z_button = 1;
  if ((byte5 >> 1) & 1)
    _c_button = 1;
  _accel_x_axis += (byte5 >> 2) & 0x03;
  _accel_y_axis += (byte5 >> 4) & 0x03;
  _accel_z_axis += (byte5 >> 6) & 0x03;
}

void NUNCHUCK::print_data_new_readings(void)
{
  // this function is called as many times as reads from the NunChuck;
  // it updates the LCD screen with the new values read from the NunChuck;
  sprintf(_text_buffer, "%03d", _joy_x_axis);
  LCD_PutStr(88, 64, (char *)_text_buffer, DEFAULT_FONT, C_YELLOW, C_BLACK);
  sprintf(_text_buffer, "%03d", _joy_y_axis);
  LCD_PutStr(88, 80, (char *)_text_buffer, DEFAULT_FONT, C_YELLOW, C_BLACK);
  sprintf(_text_buffer, "%04d", _accel_x_axis);
  LCD_PutStr(88, 96, (char *)_text_buffer, DEFAULT_FONT, C_YELLOW, C_BLACK);
  sprintf(_text_buffer, "%04d", _accel_y_axis);
  LCD_PutStr(88, 112, (char *)_text_buffer, DEFAULT_FONT, C_YELLOW, C_BLACK);
  sprintf(_text_buffer, "%04d", _accel_z_axis);
  LCD_PutStr(88, 128, (char *)_text_buffer, DEFAULT_FONT, C_YELLOW, C_BLACK);
  sprintf(_text_buffer, "%01d", _z_button);
  LCD_PutStr(88, 144, (char *)_text_buffer, DEFAULT_FONT, C_YELLOW, C_BLACK);
  sprintf(_text_buffer, "%01d", _c_button);
  LCD_PutStr(88, 160, (char *)_text_buffer, DEFAULT_FONT, C_YELLOW, C_BLACK);
}
